-------------------------------------------------------------------------------
--=> weizPVP.lua
-- weizPVP CORE
-------------------------------------------------------------------------------
--> LuaCheck Settings
-- luacheck: no max line length
-- luacheck: ignore  _.* for k self
--> DEBUG GLOBALS
-- luacheck: globals ViragDevTool_AddData
--> weizPVP Globals
-- luacheck: globals weizPVP
--> Libs
-- luacheck: globals LibStub
--> WoW Globals
-- luacheck: globals table.wipe GetRealmName GetUnitName UnitLevel
-------------------------------------------------------------------------------
--=> LIBS
-------------------------------------------------------------------------------
weizPVP = LibStub("AceAddon-3.0"):NewAddon("weizPVP", "AceConsole-3.0", "AceEvent-3.0", "AceTimer-3.0")
local SM = LibStub:GetLibrary("LibSharedMedia-3.0")
local miniMapIcon = LibStub("LibDBIcon-1.0")
local weizPVP_LDB = LibStub("LibDataBroker-1.1"):NewDataObject("weizPVPMM", {
	type = "data source",
	text = "weizPVP",
	icon = "Interface\\Addons\\weizPVP\\Icons\\wpvp_icon.tga",
	OnClick = function() weizPVP:MiniMapToggle() end
})
local mmIcon = LibStub("LibDBIcon-1.0")


-------------------------------------------------------------------------------
--=> GLOBALS TO LOCALS
-------------------------------------------------------------------------------
local table_wipe = table.wipe

-------------------------------------------------------------------------------
--=> GLOBALS
-------------------------------------------------------------------------------
--> Initialize Data Tables
weizPVP.Settings = {} -- Create blank global table for weizPVP Settings
weizPVP.PlayerDB = {} -- player db of detected players
weizPVP.PlayerActiveCache = {} -- Cache of players for this session

--> Player Counts
weizPVP.NearbyCount = 0
weizPVP.ListCount = 0
weizPVP.ListVisableCount = 0

-------------------------------------------------------------------------------
--=> Minimap Icon
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
--=> Default Settings
-------------------------------------------------------------------------------
weizPVP.Settings = {
    Enabled = true,
    Shown = true,
    --> BG/BORDER FOR ALL FRAMES
    BACKGROUNDTEXTURE = "weizPVP-Background",
    BORDERTEXTURE = "weizPVP-Border",
    BACKGROUNDCOLOR = {
        R = 0,
        G = 0,
        B = 0,
        A = 0.7
    },
    --> MAIN FRAME
    MAIN = {
        WIDTH = 260,
        HEIGHT = 0,
        BORDER_SIZE = 2
    },
    --> SCROLL FRAME
    SCROLL = {
        HEIGHT = 0,
        WIDTH = 0,
        scrollBarWidth = 4,
        SCROLLOFFET = 0
    },
    --> LIST FRAME
    LIST = {
        ROWHEIGHT = 0,
        HEIGHT = 0,
        WIDTH = 0
    },
    --> Header Settings
    HEADER = {
        HEIGHT = 34,
        WIDTH = 0,
        TEXT = {
            FONT = "Roboto Condensed BoldItalic",
            FONTSIZE = 20,
            FONTOUTLINE = "OUTLINE"
        },
        BUTTONS = {
            CONFIG = true,
            STATS = true,
            CLEAR = true,
            PREV = true,
            NEXT = true,
            CLOSE = true
        }
    },
    --> FOOTER FRAME
    FOOTER = {
        HEIGHT = 22,
        TEXT = {
            -- FONT = "Accidental Presidency",
            FONT = "Roboto Condensed Bold",
            FONTSIZE = 16,
            FONTOUTLINE = "OUTLINE"
        }
    },
    --> BARS
    BARS = {
        ALPHA_DEFAULT = 1,
        ALPHA_INACTIVE = 0.5,
        ALPHA_DEAD = 0.7,
        MAXNUMBER = 30,
        HEIGHT = 18,
        WIDTH = 0,
        SPACING = 1,
        NUM_DISPLAYED = 12,
        TEXTURE = "weizPVP.Bar",
        HIGHLIGHTTEXTURE = "weizPVP.Bar-Highlight",
        BGTEXTURE = "weizPVP.BarBackground",
        NAME_FONT_DIFF = -2,
        NAME_ALPHA_STEALTH = 0.6,
        LEVEL_FONT_DIFF = -2,
        GUILD_FONT_DIFF = -4
    },
    --> FONTS
    FONTS = {
        NARROW = "Accidental Presidency",
        NORMAL = "Roboto Condensed Bold",
        BOLD = "Roboto Condensed Bold",
        ITALIC = "Accidental Presidency Italic",
        BOLDITALIC = "Roboto Condensed BoldItalic"
    }
}

--=> Computed Settings
-------------------------------------------------------------------------------
weizPVP.Settings.LIST.ROWHEIGHT = weizPVP.Settings.BARS.HEIGHT + weizPVP.Settings.BARS.SPACING
weizPVP.Settings.SCROLL.HEIGHT =
    (weizPVP.Settings.LIST.ROWHEIGHT * weizPVP.Settings.BARS.NUM_DISPLAYED) + weizPVP.Settings.BARS.SPACING
weizPVP.Settings.SCROLL.WIDTH = weizPVP.Settings.MAIN.WIDTH
weizPVP.Settings.MAIN.HEIGHT = weizPVP.Settings.SCROLL.HEIGHT
weizPVP.Settings.LIST.HEIGHT = weizPVP.Settings.LIST.ROWHEIGHT * weizPVP.Settings.BARS.MAXNUMBER
weizPVP.Settings.LIST.WIDTH = weizPVP.Settings.SCROLL.WIDTH
weizPVP.Settings.BARS.WIDTH = weizPVP.Settings.LIST.WIDTH
weizPVP.Settings.HEADER.WIDTH = weizPVP.Settings.MAIN.WIDTH

-------------------------------------------------------------------------------
--=> LIB SHARED MEDIA ADDTIONS
-------------------------------------------------------------------------------
--> Status Bars
SM:Register("statusbar", "weizPVP.Bar", [[Interface\Addons\weizPVP\Textures\bar.tga]])
SM:Register("statusbar", "weizPVP.Bar-Highlight", [[Interface\Addons\weizPVP\Textures\bar-highlight.tga]])
--> Bakgrounds
SM:Register("background", "weizPVP-Background", [[Interface\Addons\weizPVP\Textures\bg.tga]])
SM:Register("background", "weizPVP.BarBackground", [[Interface\Addons\weizPVP\Textures\bar_bg.tga]])
--> Borders
SM:Register("border", "weizPVP-Border", [[Interface\Addons\weizPVP\Textures\border.tga]])
--> Fonts
SM:Register("font", "Roboto Condensed Bold", [[Interface\Addons\weizPVP\Fonts\RobotoCondensed-Bold.ttf]])
SM:Register("font", "Roboto Condensed BoldItalic", [[Interface\Addons\weizPVP\Fonts\RobotoCondensed-BoldItalic.ttf]])
SM:Register("font", "Pepsi", [[Interface\Addons\weizPVP\Fonts\Pepsi.ttf]])
SM:Register("font", "Accidental Presidency", [[Interface\Addons\weizPVP\Fonts\Accidental-Presidency.ttf]])
SM:Register("font", "Accidental Presidency Italic", [[Interface\Addons\weizPVP\Fonts\Accidental-Presidency-Italic.ttf]])
SM:Register("font", "Open Sans Condensed", [[Interface\Addons\weizPVP\Fonts\OpenSansCondensed.ttf]])
SM:Register("font", "Open Sans Condensed Bold", [[Interface\Addons\weizPVP\Fonts\OpenSans-CondBold.ttf]])
SM:Register("font", "Ubuntu Condensed", [[Interface\Addons\weizPVP\Fonts\Ubuntu-C.ttf]])
SM:Register("font", "YanoneKaffeesatz Regular", [[Interface\Addons\weizPVP\Fonts\YanoneKaffeesatz-Regular.ttf]])
--> Sounds
--> Sound Alarms
SM:Register("sound", "alarm-1", [[Interface\Addons\weizPVP\Sounds\alarm-1.ogg]])
SM:Register("sound", "alarm-2", [[Interface\Addons\weizPVP\Sounds\alarm-2.ogg]])
--> Sound Alerts
SM:Register("sound", "alert-1", [[Interface\Addons\weizPVP\Sounds\alert-1.ogg]])
SM:Register("sound", "alert-2", [[Interface\Addons\weizPVP\Sounds\alert-2.ogg]])
SM:Register("sound", "alert-3", [[Interface\Addons\weizPVP\Sounds\alert-3.ogg]])
SM:Register("sound", "alert-4", [[Interface\Addons\weizPVP\Sounds\alert-4.ogg]])
SM:Register("sound", "alert-5", [[Interface\Addons\weizPVP\Sounds\alert-5.ogg]])
SM:Register("sound", "alert-6", [[Interface\Addons\weizPVP\Sounds\alert-6.ogg]])
SM:Register("sound", "alert-7", [[Interface\Addons\weizPVP\Sounds\alert-7.ogg]])
SM:Register("sound", "alert-8", [[Interface\Addons\weizPVP\Sounds\alert-8.ogg]])
SM:Register("sound", "alert-9", [[Interface\Addons\weizPVP\Sounds\alert-9.ogg]])
SM:Register("sound", "alert-10", [[Interface\Addons\weizPVP\Sounds\alert-10.ogg]])
SM:Register("sound", "alert-11", [[Interface\Addons\weizPVP\Sounds\alert-11.ogg]])
SM:Register("sound", "alert-12", [[Interface\Addons\weizPVP\Sounds\alert-12.ogg]])
SM:Register("sound", "alert-13", [[Interface\Addons\weizPVP\Sounds\alert-13.ogg]])
SM:Register("sound", "alert-14", [[Interface\Addons\weizPVP\Sounds\alert-14.ogg]])
--> Sound Alarms
SM:Register("sound", "beep-1", [[Interface\Addons\weizPVP\Sounds\beep-1.ogg]])
SM:Register("sound", "beep-2", [[Interface\Addons\weizPVP\Sounds\beep-2.ogg]])
--> Sound Interface
SM:Register("sound", "interface-add", [[Interface\Addons\weizPVP\Sounds\interface-add.ogg]])
SM:Register("sound", "interface-remove", [[Interface\Addons\weizPVP\Sounds\binterface-remove.ogg]])
--> Sound Lockon
SM:Register("sound", "lockon-1", [[Interface\Addons\weizPVP\Sounds\lockon-1.ogg]])
--> Sound Music
SM:Register("sound", "music-lmfoa", [[Interface\Addons\weizPVP\Sounds\music-lmfoa.ogg]])
SM:Register("sound", "music-pistolero", [[Interface\Addons\weizPVP\Sounds\music-pistolero.ogg]])
--> Sound Notice
SM:Register("sound", "notice-1", [[Interface\Addons\weizPVP\Sounds\notice-1.ogg]])
SM:Register("sound", "notice-2", [[Interface\Addons\weizPVP\Sounds\notice-2.ogg]])
SM:Register("sound", "notice-3", [[Interface\Addons\weizPVP\Sounds\notice-3.ogg]])
--> Sound Positive
SM:Register("sound", "positive-1", [[Interface\Addons\weizPVP\Sounds\positive-1.ogg]])
--> Sound Voices
SM:Register("sound", "voice-aggro", [[Interface\Addons\weizPVP\Sounds\voice-aggro.ogg]])

--=> OnInitialize
-------------------------------------------------------------------------------
function weizPVP:OnInitialize()
    --> LOAD ACE3 DB
    self.db = LibStub("AceDB-3.0"):New("weizPVP_DB")

    --> Initialize DBs & Lookup Tables
    weizPVP:CreateAbilityDB()
    weizPVP:CreateSpecDB()

    --> Set Global Player Info
    weizPVP.PlayerLevel = UnitLevel("player") -- player's level
    weizPVP.CurrentRealm = GetRealmName() -- player's realm name
    weizPVP.CurrentCharName = GetUnitName("player") -- player's char name

    --> Minimap Icon
    self.db = LibStub("AceDB-3.0"):New("weizPVP_DB", {
        profile = {
            minimap = {
                hide = false,
            },
        },
    })
    mmIcon:Register("weizPVP", weizPVP_LDB, self.db.profile.minimap)
    self:RegisterChatCommand("weizPVPMM", "CommandMiniMapIcon")
end

function weizPVP:CommandMiniMapIcon()
	self.db.profile.minimap.hide = not self.db.profile.minimap.hide
	if self.db.profile.minimap.hide then
		miniMapIcon:Hide("weizPVPMM!")
	else
		miniMapIcon:Show("weizPVPMM!")
	end
end

--> OnLoad
-------------------------------------------------------------------------------
function weizPVP:OnLoad()
end

--=> OnEnable
-------------------------------------------------------------------------------
function weizPVP:OnEnable()
    --> Get PlayerCache DB
    if self.db.profile.PlayerDB then
        weizPVP.PlayerDB = self.db.profile.PlayerDB
    end

    --> Setup Timers
    weizPVP:ScheduleRepeatingTimer("ManageExpirations", 1)
    -- weizPVP:ScheduleRepeatingTimer("GarbageCollect", 16)
    weizPVP:ScheduleRepeatingTimer("SaveAddonData", 10)

    --> Setup Registered Events
    weizPVP:RegisterEvent("ZONE_CHANGED", "ZoneChangedEvent")
    weizPVP:RegisterEvent("ZONE_CHANGED_INDOORS", "ZoneChangedEvent")
    weizPVP:RegisterEvent("ZONE_CHANGED_NEW_AREA", "ZoneChangedNewAreaEvent")
    weizPVP:RegisterEvent("PLAYER_ENTERING_WORLD", "PlayerEnteringWorldEvent")
    weizPVP:RegisterEvent("UNIT_FACTION", "ZoneChangedEvent")
    weizPVP:RegisterEvent("PLAYER_TARGET_CHANGED", "PlayerTargetEvent")
    weizPVP:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED", "CombatLogEvent")
    weizPVP:RegisterEvent("PLAYER_REGEN_ENABLED", "LeftCombatEvent")
    weizPVP:RegisterEvent("PLAYER_DEAD", "PlayerDeadEvent")
    weizPVP:RegisterEvent("CHAT_MSG_CHANNEL_NOTICE", "ChannelNoticeEvent")
    weizPVP:RegisterEvent("UPDATE_MOUSEOVER_UNIT", "PlayerMouseoverEvent")
    -- weizPVP:RegisterEvent("UNIT_HEALTH_FREQUENT", "UnitHealth")
    weizPVP:RegisterEvent("UNIT_HEALTH", "UnitHealth")
    weizPVP:RegisterEvent("NAME_PLATE_UNIT_ADDED", "NameplateAdded")
    weizPVP:RegisterEvent("NAME_PLATE_UNIT_REMOVED", "NameplateRemoved")
    weizPVP:RegisterEvent("PLAYER_REGEN_DISABLED", "EnteringCombat")
    weizPVP:RegisterEvent("PLAYER_REGEN_ENABLED", "LeavingCombat")
    weizPVP:RegisterEvent("PLAYER_LEVEL_UP", "PlayerLevelUp")
end

--=> OnDisable
-------------------------------------------------------------------------------
function weizPVP:OnDisable()
    weizPVP:UnregisterEvent("ZONE_CHANGED", "ZoneChangedEvent")
    weizPVP:UnregisterEvent("ZONE_CHANGED_INDOORS", "ZoneChangedEvent")
    weizPVP:UnregisterEvent("ZONE_CHANGED_NEW_AREA", "ZoneChangedNewAreaEvent")
    weizPVP:UnregisterEvent("PLAYER_ENTERING_WORLD", "PlayerEnteringWorldEvent")
    weizPVP:UnregisterEvent("UNIT_FACTION", "ZoneChangedEvent")
    weizPVP:UnregisterEvent("PLAYER_TARGET_CHANGED", "PlayerTargetEvent")
    weizPVP:UnregisterEvent("UPDATE_MOUSEOVER_UNIT", "PlayerMouseoverEvent")
    weizPVP:UnregisterEvent("COMBAT_LOG_EVENT_UNFILTERED", "CombatLogEvent")
    weizPVP:UnregisterEvent("PLAYER_REGEN_ENABLED", "LeftCombatEvent")
    weizPVP:UnregisterEvent("PLAYER_DEAD", "PlayerDeadEvent")
    weizPVP:UnregisterEvent("CHAT_MSG_CHANNEL_NOTICE", "ChannelNoticeEvent")
    -- weizPVP:UnregisterEvent("UNIT_HEALTH_FREQUENT", "UnitHealth")
    weizPVP:UnregisterEvent("UNIT_HEALTH", "UnitHealth")
    weizPVP:UnregisterEvent("NAME_PLATE_UNIT_ADDED", "NameplateAdded")
    weizPVP:UnregisterEvent("NAME_PLATE_UNIT_REMOVED", "NameplateRemoved")
    weizPVP:UnregisterEvent("PLAYER_REGEN_DISABLED", "EnteringCombat")
    weizPVP:UnregisterEvent("PLAYER_REGEN_ENABLED", "LeavingCombat")
    weizPVP:UnregisterEvent("PLAYER_LEVEL_UP", "PlayerLevelUp")
end

--=> Clear List Data
-------------------------------------------------------------------------------
function weizPVP:ClearListData()
    table_wipe(weizPVP.PlayerActiveCache)
    table_wipe(weizPVP.ActiveList)
    table_wipe(weizPVP.InactiveList)
    table_wipe(weizPVP.NearbyList)
    table_wipe(weizPVP.CurrentList)
    weizPVP:RefreshCurrentList()
    weizPVP:UpdateNearbyCount()
end

--=> Save Addon Data
-------------------------------------------------------------------------------
function weizPVP:SaveAddonData()
    self.db.profile.Settings = weizPVP.Settings
    self.db.profile.PlayerDB = weizPVP.PlayerDB
end

--=> MiniMap Toggle
-------------------------------------------------------------------------------
local mmIconVisable = true
function weizPVP:MiniMapToggle()
    if mmIconVisable == true then
        weizPVP.HeaderFrame:Hide()
        weizPVP.CoreFrame:Hide()
        mmIconVisable = false
    else
        weizPVP.HeaderFrame:Show()
        weizPVP.CoreFrame:Show()
        mmIconVisable = true

    end
end
